/**
 * Calculate the # of periods to break-even on an initial investment,
 * given some array of cash flows for as many periods.
 *
 * Returns -1 if never break even.
 */
function payback_period(initial, flows = []) {
    let balance = -initial;
    let periods = 0;

    for ( const flow of flows ) {
        console.log({flow, balance, periods})
        if ( balance + flow > 0 ) {
            periods += (Math.abs(balance)) / flow;
            return periods;
        } else {
            balance += flow;
            periods += 1;
        }
    }

    return -1;
}

/**
 * Calculate the portfolio weights (roughtly) required to reach a target
 * beta for a portfolio composed of 2 stocks, given their betas.
 */
function portfolio_weights(stock_a_beta, stock_b_beta, target_beta = 1, resolution = 0.0001) {
    let stock_a_weight = 1
    let stock_b_weight = 0

    let target = (a_w, b_w) => (stock_a_beta * a_w) + (stock_b_beta * b_w)
    let roughly = value => (value < (target_beta + (2 * resolution))) && (value > (target_beta - (2 * resolution)))
    while ( stock_a_weight >= 0 && !roughly(target(stock_a_weight, stock_b_weight)) ) {
        stock_b_weight += resolution
        stock_a_weight -= resolution
    }

    if ( stock_a_weight < 0 ) return false
    return { stock_a_weight, stock_b_weight }
}

module.exports = exports = { payback_period, portfolio_weights }
